/* 
 * File   : ffield.hpp
 * Version: 2008-06-06
 *
 * Copyright (C) 2008 Jochen Peters
 *
 * dev@emirp.net
 * http://www.emirp.net/
 * 
 * This file is part of FissionField and FissionField2x.
 *
 * FissionField(2x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * FissionField(2x) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with FissionField(2x). If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef ATOMVS
#define ATOMVS

namespace ffield {
	
	enum color { RED=0, BLUE=1, NONE=9 };
	
	class Atom {
	
		public:
			Atom(int maxNumParticles);
			void reset();
			int getNumParticles() const;
			int getMaxNumParticles() const;
			enum color getTeam() const;
			int addParticle(enum color team);
	
		protected:
			int maxNumParticles;
			int numParticles;
			enum color team;
	
	};
	
	class AtomicField {
		
		public:
			AtomicField(int size);
			~AtomicField();
			AtomicField(const AtomicField& field);
			Atom* getAtom(int row, int col) const;
			int getSize() const;
			int getNumMoves() const;
			int getScore(enum color team) const;
			int getNumParticles(enum color team) const;
			bool okayToAdd(int row, int col, enum color team) const;
			bool makeMove(int row, int col, enum color team);
			enum color checkForWinner();
			void printField() const;
	
		protected:
			Atom*** atoms;
			int size;
			int numMoves;
			int score[2];
			virtual void addUpdate(int row, int col);
			virtual void explodeUpdate(int row, int col, enum color team, int phase);
	
		private:
			bool validCoords(int row, int col) const;
			int addParticle(int row, int col, enum color team);
			void updateScore();
	
	};

}

#endif

