/* 
 * File   : ffield_player.hpp
 * Version: 2008-06-06
 *
 * Copyright (C) 2008 Jochen Peters
 *
 * dev@emirp.net
 * http://www.emirp.net/
 * 
 * This file is part of FissionField and FissionField2x.
 *
 * FissionField(2x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * FissionField(2x) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with FissionField(2x). If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "ffield.hpp"

namespace ffield {
	
	class AtomvsPlayer { // abstract
		
		public:
			AtomvsPlayer(enum color team);
			~AtomvsPlayer();

			int getNumValidMoves(const AtomicField* actualField);
			void getValidMoves(const AtomicField* actualField, int* validMoves_row, int* validMoves_col);

			virtual void makeMove(const AtomicField* actualField, int* moveToRow, int* moveToCol) = 0; // abstract

		protected:
			enum color team;

	};


	class EvalPlayer : public AtomvsPlayer { // abstract
		public:
			EvalPlayer(enum color team);
			virtual void makeMove(const AtomicField* actualField, int* moveToRow, int* moveToCol);

		protected:
			virtual float evaluateMove(const AtomicField* actualField, int row, int col) = 0; // abstract

	};


	class RandomPlayer : public AtomvsPlayer {
		public:
			RandomPlayer(enum color team);
			virtual void makeMove(const AtomicField* actualField, int* moveToRow, int* moveToCol);
	};

	class MaxFieldsPlayer : public EvalPlayer {
		public:
			MaxFieldsPlayer(enum color team);

		protected:
			virtual float evaluateMove(const AtomicField* actualField, int row, int col);

	};

	class MinFieldsPlayer : public EvalPlayer {
		public:
			MinFieldsPlayer(enum color team);

		protected:
			virtual float evaluateMove(const AtomicField* actualField, int row, int col);

	};

	class MaxParticlesPlayer : public EvalPlayer {
		public:
			MaxParticlesPlayer(enum color team);

		protected:
			virtual float evaluateMove(const AtomicField* actualField, int row, int col);

	};

}

