/* 
 * File   : sdl_ffield.cpp
 * Version: 2008-06-06
 *
 * Copyright (C) 2008 Jochen Peters
 *
 * dev@emirp.net
 * http://www.emirp.net/
 * 
 * This file is part of FissionField and FissionField2x.
 *
 * FissionField(2x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * FissionField(2x) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with FissionField(2x). If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <iostream>
#include <SDL/SDL.h>
//#include <SDL/SDL_audio.h>
#include <SDL/SDL_mixer.h>
#include "sdl_ffield.hpp"

/******************************
 * Sound callback function    *
 ******************************/
/*
void mixAudio(void* unused, Uint8* stream, int len) {
	
	Uint32 amount = (soundToPlay.len-soundToPlay.pos);
	if(amount > len) {
		amount = len;
	}
	SDL_MixAudio(stream, &soundToPlay.data[soundToPlay.pos], amount, SDL_MIX_MAXVOLUME);
	soundToPlay.pos += amount;
}
*/


/******************************
 * sdlAtomicField             *
 ******************************/

sdlAtomicField::sdlAtomicField(int size, Uint8 xpos, Uint8 ypos, bool sound) : AtomicField(size) {

	this->ulCornerX = xpos;
	this->ulCornerY = ypos;
	this->playSound = sound;

	/* Setup VIDEO */
	/* Load images */
	atomBitmaps[0] = SDL_LoadBMP("bitmaps/n0.bmp");
	atomBitmaps[1] = SDL_LoadBMP("bitmaps/r1.bmp");
	atomBitmaps[2] = SDL_LoadBMP("bitmaps/r2.bmp");
	atomBitmaps[3] = SDL_LoadBMP("bitmaps/r3.bmp");
	atomBitmaps[4] = SDL_LoadBMP("bitmaps/r4.bmp");
	atomBitmaps[5] = SDL_LoadBMP("bitmaps/b1.bmp");
	atomBitmaps[6] = SDL_LoadBMP("bitmaps/b2.bmp");
	atomBitmaps[7] = SDL_LoadBMP("bitmaps/b3.bmp");
	atomBitmaps[8] = SDL_LoadBMP("bitmaps/b4.bmp");

	/* Check loading */
	for(int i=0; i<9; i++) {
		if(atomBitmaps[i] == NULL) {
			std::cerr << "Bitmap file(s) missing. Abort." << std::endl;
			exit(1);
		}
	}

	/* Set up blitting rectangle */
	image_rect.w = atomBitmaps[0]->w;
	image_rect.h = atomBitmaps[0]->h;


	/* Setup AUDIO */
	if(sound) {
		/* Load sound file
	    SDL_AudioSpec wave;
	    Uint8 *data;
	    Uint32 dlen;
	    if(SDL_LoadWAV("sound/click.wav", &wave, &data, &dlen) == NULL) {
			std::cerr << "Sound file(s) missing. Abort." << std::endl;
			exit(1);
	    }
		*/
	
		/* Convert loaded sound data to 16-bit stereo, 22kHz
	    SDL_AudioCVT cvt;
	    SDL_BuildAudioCVT(&cvt, wave.format, wave.channels, wave.freq, AUDIO_S16, 2, 22050);
	    cvt.buf = (Uint8*) malloc(dlen*cvt.len_mult);
	    memcpy(cvt.buf, data, dlen);
	    cvt.len = dlen;
	    SDL_ConvertAudio(&cvt);
	    SDL_FreeWAV(data);
		*/
	
		/* Keep converted data in class-variable
		this->clickSound.pos = 0;
		this->clickSound.len = cvt.len_cvt;
	    this->clickSound.data = cvt.buf;
		*/

		this->clickSound = Mix_LoadWAV("sound/click.wav");
		if(this->clickSound == NULL) {
			std::cerr << "Sound file(s) missing. Abort." << std::endl;
			exit(1);
		}

	}

	/* Display field */
	updateField();

}


sdlAtomicField::~sdlAtomicField() {

	if(this->playSound) {
		Mix_FreeChunk(this->clickSound);
	}

}

int sdlAtomicField::getRow(int posY) const {

	int row;
	int h = atomBitmaps[0]->h;
	int pos = posY - ulCornerY;

	if(pos < 0 || pos > h*this->getSize()) {
		row = -1;
	}
	else {
		row = (int) (pos / h);
	}

	return row;
}

int sdlAtomicField::getCol(int posX) const {

	int col;
	int w = atomBitmaps[0]->w;
	int pos = posX - ulCornerX;

	if(pos < 0 || pos > w*this->getSize()) {
		col = -1;
	}
	else {
		col = (int) (pos / w);
	}

	return col;
}

int sdlAtomicField::getPosX(int col) const {
	return col*atomBitmaps[0]->w + ulCornerX;
}

int sdlAtomicField::getPosY(int row) const {
	return row*atomBitmaps[0]->h + ulCornerY;
}

void sdlAtomicField::updateField(int row, int col) {

	/* Get Atom information */
	ffield::Atom* tempAtom = this->getAtom(row, col);
	enum color team = tempAtom->getTeam();
	int numParticles = tempAtom->getNumParticles();

	/* Set position */
	image_rect.x = getPosX(col);
	image_rect.y = getPosY(row);

	/* Set bitmap */
	if(numParticles == 0) {
		SDL_BlitSurface(atomBitmaps[0], NULL, screen, &image_rect);
	}
	else {
		SDL_BlitSurface(atomBitmaps[team*4+numParticles], NULL, screen, &image_rect);
	}

	/* Display */
	SDL_Flip(screen);

}

void sdlAtomicField::updateField() {

	for(int i=0; i<this->getSize(); i++) {
		for(int j=0; j<this->getSize(); j++) {
			updateField(i, j);
		}
	}

}

void sdlAtomicField::addUpdate(int row, int col) {

	this->updateField(row, col);

	/* Sound */
	if(this->playSound) {
		//std::cout << '\a' << std::flush; // beep
		/* Put sound data in soundToPlay 
    	SDL_LockAudio();
	    soundToPlay.data = clickSound.data;
	    soundToPlay.len = clickSound.len;
	    soundToPlay.pos = 0;
	    SDL_UnlockAudio();
		*/
		Mix_PlayChannel(-1, this->clickSound, 0);
	}

	/* Delay */
	SDL_Delay(DEFAULT_DELAY_TIME);

}

void sdlAtomicField::explodeUpdate(int row, int col, enum color team, int phase) {

	/* Set bitmap */
	if(phase == 4) {
		/* Set position */
		image_rect.x = getPosX(col);
		image_rect.y = getPosY(row);
		/* Display */
		//SDL_BlitSurface(explodeBitmaps[team], NULL, screen, &image_rect);
		ffield::Atom* tempAtom = this->getAtom(row, col);
		int maxNumParticles = tempAtom->getMaxNumParticles();
		SDL_BlitSurface(atomBitmaps[team*4+maxNumParticles], NULL, screen, &image_rect);
		SDL_Flip(screen);
	}
	else {
		updateField(row, col);
	}

}


/******************************
 * sdlFieldCursor             *
 ******************************/

sdlFieldCursor::sdlFieldCursor(const sdlAtomicField* field) {
	
	this->field = field;
	this->posRow = 0;
	this->posCol = 0;
	this->visible = false;

	/* Load bitmaps */
	this->activeFrame = SDL_LoadBMP("bitmaps/activeframe.bmp");
	this->inactiveFrame = SDL_LoadBMP("bitmaps/inactiveframe.bmp");
	/* Check loading */
	if(activeFrame==NULL || inactiveFrame==NULL) {
		std::cerr << "Bitmap file(s) missing. Abort." << std::endl;
		exit(1);
	}
	/* Set transparent color */
	SDL_SetColorKey(activeFrame, SDL_SRCCOLORKEY, SDL_MapRGB(activeFrame->format, 255, 0, 255));
	SDL_SetColorKey(inactiveFrame, SDL_SRCCOLORKEY, SDL_MapRGB(activeFrame->format, 255, 0, 255));

	/* Set up blitting rectangle */
	image_rect.w = activeFrame->w;
	image_rect.h = activeFrame->h;

}

void sdlFieldCursor::moveTo(int row, int col) {
	
	/* Check if new position is valid */
	if(row < 0 || col < 0 || row >= field->getSize() || col >= field->getSize()) {
		return;
	}

	bool isVisible = this->visible;

	/* Update display */
	if(isVisible) {
		setVisible(false);
	}

	/* Save new position */
	this->posRow = row;
	this->posCol = col;

	/* Update display */
	if(isVisible) {
		setVisible(true);
	}

}

void sdlFieldCursor::moveRight() {
	if(posCol >= field->getSize()-1) {
		moveTo(posRow, 0);
	}
	else {
		moveTo(posRow, posCol+1);
	}
}

void sdlFieldCursor::moveDown() {
	if(posRow >= field->getSize()-1) {
		moveTo(0, posCol);
	}
	else {
		moveTo(posRow+1, posCol);
	}
}

void sdlFieldCursor::moveLeft() {
	if(posCol <= 0) {
		moveTo(posRow, field->getSize()-1);
	}
	else {
		moveTo(posRow, posCol-1);
	}
}

void sdlFieldCursor::moveUp() {
	if(posRow <= 0) {
		moveTo(field->getSize()-1, posCol);
	}
	else {
		moveTo(posRow-1, posCol);
	}
}

void sdlFieldCursor::setVisible(bool seton) {

	/* Set position */
	image_rect.x = field->getPosX(posCol);
	image_rect.y = field->getPosY(posRow);

	/* Update display */
	if(seton) { // turn on
		SDL_BlitSurface(activeFrame, NULL, screen, &image_rect);
	}
	else { // turn off
		SDL_BlitSurface(inactiveFrame, NULL, screen, &image_rect);
	}
	SDL_Flip(screen);

	this->visible = seton;

}

int sdlFieldCursor::getRow() const {
	return posRow;
}

int sdlFieldCursor::getCol() const {
	return posCol;
}

