/* 
 * File   : sdl_textmenu.hpp
 * Version: 2008-06-06
 *
 * Copyright (C) 2008 Jochen Peters
 *
 * dev@emirp.net
 * http://www.emirp.net/
 * 
 * This file is part of FissionField and FissionField2x.
 *
 * FissionField(2x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * FissionField(2x) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with FissionField(2x). If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <SDL/SDL.h>
#include "SFont.h"

#define SDL_TEXTMENU_MAXLENGTH 256

typedef struct {
	char text[SDL_TEXTMENU_MAXLENGTH];
	unsigned int x;
	unsigned int y;
	unsigned int w;
	unsigned int h;
} sdlTextMenuItem;


class sdlTextMenu {
	
	public:
		sdlTextMenu(SDL_Surface* _screen, int _numItems, int _activeIndex, const char* _fontFile, Uint8 _activeR=255, Uint8 _activeG=204, Uint8 _activeB=0, Uint8 _inactiveR=0, Uint8 _inactiveG=0, Uint8 _inactiveB=0);
		~sdlTextMenu();

		void setItem(int _index, char* _text, int _posX, int _posY);
		void displayMenu();
		void updateItem(int _index, char* _text, int _posX, int _posY, Uint32 _bgcolor);
		void updateItem(int _index, char* _text, Uint32 _bgcolor);

		int getActiveIndex();
		void activate(int _index);
		void activatePrev();
		void activateNext();

	protected:
		SDL_Surface* pScreen;
		SFont_Font* font;
		sdlTextMenuItem* items;
		int numItems;
		int activeIndex;
		Uint32 inactiveColor;
		Uint32 activeColor;

		void displayItem(int _index);
		void clearItem(int _index, Uint32 _bgcolor);
		void colorItem(int _index, bool _active);
		static void replaceColor(SDL_Surface* _surf, SDL_Rect* _area, Uint32 _fromColor, Uint32 _toColor);

};

